# Generate script to use Leaflet library
# $Id$
# Author: gcc@fargos.net http://www.fargos.net/gcc.html

# INCLUDE_REGIONS - filter on specified regions
# ALL_KNOWN_REGIONS - generate links for all regions;
#   defaults to INCLUDE_REGIONS if no value specified
# USE_THUMBNAILS - {0|1} indicate if thumbnail markers should be used
# 
# consume CSV lines like:
# Japan,/storage/upnp/pictures/Fixed_JapanSeoul2018/D7508689.JPG,35.5175075629575,139.568103323546,"LaLaport Yokohama"

# Add \\ prefix to special characters
function escapeSpecialCharacter(textString, special, _i, left)
{
#	print "escape " textString " len=" length(textString) > "/dev/stderr";
	_i = index(textString, special);
	if (_i == 0) {
		return (textString);
	}
	result = substr(textString, 1, _i - 1) "\\" special;
	left = substr(textString, _i + 1);
	while (length(left) > 0) {
		_i = index(left, special);
		if (_i == 0) {
			result = result left;
#	print "result=" result > "/dev/stderr";
			return (result);
		}
		result = result substr(left, 1, _i - 1) "\\" special;
		left = substr(left, _i + 1);
	}
#	print "result=" result > "/dev/stderr";
	return (result);
}

function findZoomLevel(degrees, _p, _i, _d)
{
	if (degrees >= 180) {
		_p = 101;
	} else if (degrees >= 90) {
		_p = 103;
	} else if (degrees >= 0.5) {
		_p = 110;
	} else {
		_p = 120;
	}
	_d = (_p * degrees) / 100; # +100% of degree range
	_i = 18;
	while (_i > 0) {
		# for our purposes, one more zoom level works well
		if (zoomLevelTable[_i] >= _d) {
printf("zoom %d degrees=%g zoomRange=%g\n", _i, degrees, zoomLevelTable[_i]) > "/dev/stderr";
			return (_i);
		}
		_i -= 1;
	}
	return (_i);
}

function emitHTMLheader(title)
{
	printf("<!DOCTYPE html>\n");
	printf("<!-- This page was generated by makeRegionPageHTML.awk, which was written by Geoff Carpenter http://www.fargos.net/gcc.html gcc@fargos.net -->\n");
	langSpec = ENVIRON["LANG"];
	i = index(langSpec, "_");
	if (i != 0) {
		langSpec = substr(langSpec, 1, i - 1);
	}
	if (length(langSpec) == 0) {
		langSpec = "en";
	}
	printf("<html lang=\"%s\">\n", langSpec);
	printf("<head>\n");
	printf("<title>" title "</title>\n");
	printf("  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n");
	printf("  <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\" />\n");
	printf("  <meta name=\"generator\" content=\"makeRegionPageHTML.awk - http://www.fargos.net/downloads.html#staffContent\" >\n");
	if (length(LEAFLET_VERSION) > 0) {
		hyphen = "-";
	} else {
		hyphen = "";
	}
	printf("  <link rel=\"stylesheet\" href=\"%sLeaflet%s%s/leaflet.css\"  />\n", globeRoot, hyphen, LEAFLET_VERSION);
	printf("  <script type=\"text/javascript\" src=\"%sLeaflet%s%s/leaflet.js\" ></script>\n", globeRoot, hyphen, LEAFLET_VERSION);
	printf("<style type=\"text/css\">\n");
	printf("  .linkCollection { border-style: none; border-width: 1px 1px 1px 1px; padding: 1px 1px 1px 1px; }\n");
	printf("  .regionLinks { border-style: solid; border-width: 1px 1px 1px 1px; padding: 1px 1px 1px 1px; }\n");
	printf("  #regionMap { height: 80vh; }\n");
	printf("  .popupDiv { width: auto; }\n");
	printf("  .popupThumbnail { width: 200px; float: none; }\n");
	printf("  @media (pointer: coarse) {\n");
	printf("    a { padding: 1em; }\n");
	printf("  }\n");
	printf("</style>\n");

	printf("</head>\n");
	printf("<body>\n");
	printf("<div id=\"links\" class=\"linkCollection\">\n");
	if (ROOT_PAGE_HTML != "") {
		# Add custom top-level link link
		printf("    <a href=\"%s\">%s</a>\n",
		       ROOT_PAGE_HTML, ROOT_PAGE_LABEL);
	}
	printf("  <span id=\"regionLinks\" class=\"regionLinks\">\n");
	printf("    <a href=\"%s%sglobe.html\">World Map</a>\n",
	       globeRoot, addRootPath);
	for(i=1;i<=allRegionTotal;i+=1) {
		printf("    <a href=\"%s%sregionMaps/%s.html\">%s</a>\n",
		       regionRoot, addRootPath, knownRegions[i], knownRegionTextName[i]);
	}
	printf("  </span>\n");
	printf("</div><!-- end linkCollection -->\n");
	printf("<hr />\n");
	printf("<h3>%s</h3>\n", title);
	printf("<div id=\"regionMap\"></div>\n");
}

function emitScript(provider)
{
	printf("<script>\n");
	printf("\"use strict\";\n");
	printf("var markerList = [ ];\n");
#	printf("var targetURLs = [ ];\n");
	printf("var markerTotal = 0;\n");
	printf("\n");
	printf("function getMapProvider(searchString, defaultProvider)\n");
	printf("{\n");
	printf("  var locationAttributes = searchString.substring(1).split(\"&\");\n");
	printf("  for (var i=0;i < locationAttributes.length; i += 1) {\n");
	printf("    var keyVal = locationAttributes[i].split(\"=\");\n");
	printf("    if (keyVal[0] == \"mapProvider\") {\n");
	printf("      return (decodeURIComponent(keyVal[1]));\n");
	printf("    }\n");
	printf("  }\n");
	printf("  return (defaultProvider);\n");
	printf("}\n");
	printf("\n");
	printf("var mapProviderName = getMapProvider(location.search, \"%s\");\n", provider);
	printf("\n");
	printf("\nfunction onClick(ev) {\n");
	printf("}\n\n");

	latDist = (((maxLat - minLat) * 1000) + 10) / 1000;
	longDist = (((maxLong - minLong) * 1000) + 10) / 1000;
	maxDist = latDist;
	if (longDist > maxDist) {
		maxDist = longDist;
	}
	zoomLevel = findZoomLevel(maxDist);
	midLatitude = (minLat + maxLat) / 2.0;
	midLongitude = (minLong + maxLong) / 2.0;
	
	printf("/* lat,long bounds: %g,%g - %g,%g maxDeg %g */\n",
	       minLat, minLong, maxLat, maxLong, maxDist);

	printf("var mapObj = L.map('regionMap', { center: [%g, %g], zoom: %d });\n",
	       midLatitude, midLongitude, zoomLevel);
	# TODO: emit both, do selection at runtime
	printf ("if (mapProviderName == \"google\") {\n");
	printf("  L.tileLayer('https://{s}.google.com/vt/lyrs=%s&z={z}&x={x}&y={y}', { maxZoom: 20, subdomains: ['mt0', 'mt1', 'mt2', 'mt3'] }).addTo(mapObj);\n",
		       googleMapMode);
	printf("} else { /* assume Open Street Maps */\n");
	printf("  L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { attribution: 'Map data &copy; <a href=\"https://www.openstreetmap.org/\">OpenStreetMap</a> contributors, <a href=\"https://creativecommons.org/licenses/by-sa/2.0/\">CC-BY-SA</a>' }).addTo(mapObj);\n");
	printf ("}\n");
}

function emitMarkers()
{
	printf("\n");
	for(i=1;i<=pointTotal;i+=1) {
		escapedDesc = escapeSpecialCharacter(pointDescription[i], "'");
		printf("  markerList[%d] = L.marker([%s, %s], { title: '%s' });\n",
		       i - 1, pointLatitude[i], pointLongitude[i],
		       escapedDesc);
		printf("  markerList[%d].addTo(mapObj);\n", i - 1);
		n = split(pointImage[i], pathElements, "/");
		n2 = split(pathElements[n], fileNameParts, ".");
		imageName = fileNameParts[1];
		region = pointRegion[i];
		gsub(" ", "_", region);
		if (useThumbnailMarkers == 0) {
			imageFileName = pointImage[i];
		} else {
			imageFileName = "";
			for(z=1;z<n;z+=1) {
				if (z != 1) {
					imageFileName = imageFileName pathElements[z] "/";
				} else {
					imageFileName = pathElements[z] "/";
				}
			}
			for(z=1;z<n2;z+=1) {
				if (z != 1) {
					imageFileName = imageFileName "." fileNameParts[z];
				} else {
					imageFileName = imageFileName fileNameParts[z];
				}
			}
			fileType = fileNameParts[n2];
			if ((fileType == "mp4") || (fileType == "MP4") || (fileType == "mov") || (fileType == "MOV")) {
				fileType = "jpeg";
			}
			imageFileName = imageFileName "_small." fileType;
		}
		if (dropRootPath != "") {
			z = index(imageFileName, dropRootPath);
			if (z == 1) {
printf("makeRegionPageHTML.awk: drop %s from %s ", dropRootPath, imageFileName) > "/dev/stderr";
				imageFileName = substr(imageFileName, length(dropRootPath) + 1);
printf(" to %s\n", imageFileName) > "/dev/stderr";
			}

		}

		pageURL = regionRoot addRootPath "regionMaps/" region "/imageAndMap_" imageName ".html";
		popupHTML = "<div class=\"popupDiv\">";
		popupHTML = popupHTML "<a href=\"" pageURL "\" class=\"popupLink\">" escapedDesc "</a>";
		popupHTML = popupHTML "<br />";
		popupHTML = popupHTML "<img src=\"" regionRoot imageFileName "\" class=\"popupThumbnail\">";
		popupHTML = popupHTML "</div>";
		printf("  markerList[%d].bindPopup('%s');\n", i - 1, popupHTML);
#		printf("  markerList[%d].on('dblclick', onClick);\n", i - 1);
	}
}

function emitFinalHTML()
{
	printf("  </script>\n");
	printf("</body>\n");
	printf("</html>\n");
}

BEGIN {
	# NOTE: specify Leaflet version or as the null
	# string if no version control is desired.
	LEAFLET_VERSION = "1.6";
	useThumbnailMarkers = USE_THUMBNAILS;
	dropRootPath = DROP_ROOT;
	addRootPath = ADD_ROOT;
	globeRoot = "../";
	regionRoot = "../";
	for(i=1;i<DEPTH;i+=1) {
		globeRoot = globeRoot "../";
		regionRoot = regionRoot "../";
	}

	if (addRootPath != "") {
		l = length(addRootPath);
		# add trailing "/" if not present
		if (substr(addRootPath, l, 1) != "/") {
			addRootPath = addRootPath "/";
		}
		# remove a leading "/" because we'll add one
		if (substr(addRootPath, 1, 1) == "/") {
			# absolute path
			globeRoot = "";
			regionRoot = "";
		}
	}
	googleMapMode = "s,h";
	useProvider = "openstreetmap"; # google is alternative

	allRegionsList = ALL_KNOWN_REGIONS;
	if (allRegionsList == "") {
		allRegionsList = INCLUDE_REGIONS;
	}
	allRegionTotal = split(allRegionsList, allRegions, ",");
	for(i=1;i<=allRegionTotal;i+=1) {
		region = allRegions[i];
		gsub(" ", "_", region);
		regionName = allRegions[i];
		gsub("_", " ", regionName);
		knownRegions[i] = region;
		knownRegionTextName[i] = regionName;
		printf("Add region %s -> %s\n", region, regionName) > "/dev/stderr";
	}

	regionTotal = split(INCLUDE_REGIONS, selectedRegions, ",");
	regionListText = "";
	for(i=1;i<=regionTotal;i+=1) {
		region = selectedRegions[i];
		gsub(" ", "_", region);
		regionName = selectedRegions[i];
		gsub("_", " ", regionName);
		wantedRegions[region] = regionName;
		wantedRegions[regionName] = regionName;
print "REGION: " region " " regionName > "/dev/stderr";
		if (regionListText != "") {
			regionListText = regionListText ", " regionName;
		} else {
			regionListText = regionName;
		}
	}
	gsub("_", " ", regionListText);

	minLat = 360.0;
	minLong = 360.0;
	maxLat = -360.0;
	maxLong = -360.0;

	zoomDegrees = 360.0; # tile width
	for(i=0;i<=19;i+=1) {
		# assume 3 tiles across on the screen
		if (i == 0) {
			tileCount = 1;
		} else if (i == 1) {
			tileCount = 2;
		} else {
			tileCount = 3;
		}
		zoomLevelTable[i] = zoomDegrees * tileCount;
		zoomDegrees = zoomDegrees / 2.0;
	}

	emitHTMLheader(regionListText " Sights of Sites");

	pointTotal = 0;
}

{
	region = $1;
	if (! (region in wantedRegions)) {
#		print "Ignore region: " $1 " -> " $0 > "/dev/stderr";
		next;
	}
	imageFileName = $2;
	latitude = $3;
	longitude = $4;
	description = $5;
	if (substr(description, 1, 1) == "\"") {
		description = substr(description, 2, length(description) - 2);
	}
	if ((length(latitude) == 0) || (length(longitude) == 0)) {
		print "Missing latitude/longitude: " $0 > "/dev/stderr";
		next;
	}

	if (latitude < minLat) {
		minLat = latitude;
	}
	if (latitude > maxLat) {
		maxLat = latitude;
	}
	if (longitude < minLong) {
		minLong = longitude;
	}
	if (longitude > maxLong) {
		maxLong = longitude;
	}

	pointTotal += 1;

	pointRegion[pointTotal] = region;
	pointImage[pointTotal] = imageFileName;
	pointLatitude[pointTotal] = latitude;
	pointLongitude[pointTotal] = longitude;
	pointDescription[pointTotal] = description;
}

END {
	if (pointTotal == 0) {
		printf("WARNING: %s generated zero points\n", "makeRegionPageHTML.awk") > "/dev/stderr";
	}
	emitScript(useProvider);
	emitMarkers();
	emitFinalHTML();
}

